@@ -1,16 +1,16 @@
-'''
+"""
 This module contains the execution context objects that are internal to the system.
 Not every property on these should be exposed to random Jane or Joe dagster user
 so we have a different layer of objects that encode the explicit public API
 in the user_context module
-'''
+"""
 
 from collections import namedtuple
 
 from dagster import check
-from dagster.core.definitions.executable import ExecutablePipeline
 from dagster.core.definitions.hook import HookDefinition
 from dagster.core.definitions.mode import ModeDefinition
+from dagster.core.definitions.pipeline_base import IPipeline
 from dagster.core.definitions.resource import ScopedResourcesBuilder
 from dagster.core.definitions.step_launcher import StepLauncher
 from dagster.core.errors import DagsterInvariantViolationError
@@ -24,18 +24,18 @@ from dagster.core.system_config.objects import EnvironmentConfig
 
 class SystemExecutionContextData(
     namedtuple(
-        '_SystemExecutionContextData',
+        "_SystemExecutionContextData",
         (
-            'pipeline_run scoped_resources_builder environment_config pipeline '
-            'mode_def system_storage_def intermediate_storage_def instance intermediate_storage file_manager '
-            'raise_on_error retries'
+            "pipeline_run scoped_resources_builder environment_config pipeline "
+            "mode_def system_storage_def intermediate_storage_def instance intermediate_storage file_manager "
+            "raise_on_error retries"
         ),
     )
 ):
-    '''
+    """
     SystemExecutionContextData is the data that remains constant throughout the entire
     execution of a pipeline or plan.
-    '''
+    """
 
     def __new__(
         cls,
@@ -59,28 +59,28 @@ class SystemExecutionContextData(
 
         return super(SystemExecutionContextData, cls).__new__(
             cls,
-            pipeline_run=check.inst_param(pipeline_run, 'pipeline_run', PipelineRun),
+            pipeline_run=check.inst_param(pipeline_run, "pipeline_run", PipelineRun),
             scoped_resources_builder=check.inst_param(
-                scoped_resources_builder, 'scoped_resources_builder', ScopedResourcesBuilder
+                scoped_resources_builder, "scoped_resources_builder", ScopedResourcesBuilder
             ),
             environment_config=check.inst_param(
-                environment_config, 'environment_config', EnvironmentConfig
+                environment_config, "environment_config", EnvironmentConfig
             ),
-            pipeline=check.inst_param(pipeline, 'pipeline', ExecutablePipeline),
-            mode_def=check.inst_param(mode_def, 'mode_def', ModeDefinition),
+            pipeline=check.inst_param(pipeline, "pipeline", IPipeline),
+            mode_def=check.inst_param(mode_def, "mode_def", ModeDefinition),
             system_storage_def=check.inst_param(
-                system_storage_def, 'system_storage_def', SystemStorageDefinition
+                system_storage_def, "system_storage_def", SystemStorageDefinition
             ),
             intermediate_storage_def=check.opt_inst_param(
-                intermediate_storage_def, 'intermediate_storage_def', IntermediateStorageDefinition
+                intermediate_storage_def, "intermediate_storage_def", IntermediateStorageDefinition
             ),
-            instance=check.inst_param(instance, 'instance', DagsterInstance),
+            instance=check.inst_param(instance, "instance", DagsterInstance),
             intermediate_storage=check.inst_param(
-                intermediate_storage, 'intermediate_storage', IntermediateStorage
+                intermediate_storage, "intermediate_storage", IntermediateStorage
             ),
-            file_manager=check.inst_param(file_manager, 'file_manager', FileManager),
-            raise_on_error=check.bool_param(raise_on_error, 'raise_on_error'),
-            retries=check.inst_param(retries, 'retries', Retries),
+            file_manager=check.inst_param(file_manager, "file_manager", FileManager),
+            raise_on_error=check.bool_param(raise_on_error, "raise_on_error"),
+            retries=check.inst_param(retries, "retries", Retries),
         )
 
     @property
@@ -97,13 +97,13 @@ class SystemExecutionContextData(
 
 
 class SystemExecutionContext(object):
-    __slots__ = ['_execution_context_data', '_log_manager']
+    __slots__ = ["_execution_context_data", "_log_manager"]
 
     def __init__(self, execution_context_data, log_manager):
         self._execution_context_data = check.inst_param(
-            execution_context_data, 'execution_context_data', SystemExecutionContextData
+            execution_context_data, "execution_context_data", SystemExecutionContextData
         )
-        self._log_manager = check.inst_param(log_manager, 'log_manager', DagsterLogManager)
+        self._log_manager = check.inst_param(log_manager, "log_manager", DagsterLogManager)
 
     @property
     def pipeline_run(self):
@@ -174,17 +174,17 @@ class SystemExecutionContext(object):
         return self._log_manager.logging_tags
 
     def has_tag(self, key):
-        check.str_param(key, 'key')
+        check.str_param(key, "key")
         return key in self.logging_tags
 
     def get_tag(self, key):
-        check.str_param(key, 'key')
+        check.str_param(key, "key")
         return self.logging_tags.get(key)
 
     def for_step(self, step):
         from dagster.core.execution.plan.objects import ExecutionStep
 
-        check.inst_param(step, 'step', ExecutionStep)
+        check.inst_param(step, "step", ExecutionStep)
 
         return SystemStepExecutionContext(
             self._execution_context_data, self._log_manager.with_tags(**step.logging_tags), step,
@@ -195,11 +195,11 @@ class SystemExecutionContext(object):
 
 
 class SystemPipelineExecutionContext(SystemExecutionContext):
-    __slots__ = ['_executor']
+    __slots__ = ["_executor"]
 
     def __init__(self, execution_context_data, log_manager, executor):
         super(SystemPipelineExecutionContext, self).__init__(execution_context_data, log_manager)
-        self._executor = check.inst_param(executor, 'executor', Executor)
+        self._executor = check.inst_param(executor, "executor", Executor)
 
     @property
     def executor(self):
@@ -207,13 +207,13 @@ class SystemPipelineExecutionContext(SystemExecutionContext):
 
 
 class SystemStepExecutionContext(SystemExecutionContext):
-    __slots__ = ['_step', '_resources', '_required_resource_keys', '_step_launcher']
+    __slots__ = ["_step", "_resources", "_required_resource_keys", "_step_launcher"]
 
     def __init__(self, execution_context_data, log_manager, step):
         from dagster.core.execution.plan.objects import ExecutionStep
         from dagster.core.execution.resources_init import get_required_resource_keys_for_step
 
-        self._step = check.inst_param(step, 'step', ExecutionStep)
+        self._step = check.inst_param(step, "step", ExecutionStep)
         super(SystemStepExecutionContext, self).__init__(execution_context_data, log_manager)
         self._required_resource_keys = get_required_resource_keys_for_step(
             step,
@@ -229,8 +229,8 @@ class SystemStepExecutionContext(SystemExecutionContext):
         ]
         if len(step_launcher_resources) > 1:
             raise DagsterInvariantViolationError(
-                'Multiple required resources for solid {solid_name} have inherit StepLauncher'
-                'There should be at most one step launcher resource per solid.'.format(
+                "Multiple required resources for solid {solid_name} have inherit StepLauncher"
+                "There should be at most one step launcher resource per solid.".format(
                     solid_name=step.solid_handle.name
                 )
             )
@@ -288,13 +288,13 @@ class SystemComputeExecutionContext(SystemStepExecutionContext):
 
 
 class TypeCheckContext(SystemExecutionContext):
-    '''The ``context`` object available to a type check function on a DagsterType.
+    """The ``context`` object available to a type check function on a DagsterType.
 
     Attributes:
         log (DagsterLogManager): Centralized log dispatch from user code.
         resources (Any): An object whose attributes contain the resources available to this solid.
         run_id (str): The id of this pipeline run.
-    '''
+    """
 
     def __init__(self, execution_context_data, log_manager, dagster_type):
         super(TypeCheckContext, self).__init__(execution_context_data, log_manager)
@@ -309,21 +309,24 @@ class TypeCheckContext(SystemExecutionContext):
 
 
 class HookContext(SystemExecutionContext):
-    '''The ``context`` object available to a hook function on an DagsterEvent.
+    """The ``context`` object available to a hook function on an DagsterEvent.
 
     Attributes:
         log (DagsterLogManager): Centralized log dispatch from user code.
         hook_def (HookDefinition): The hook that the context object belongs to.
-        step (ExecutionStep) The compute step associated with the hook.
-    '''
+        step (ExecutionStep): The compute step associated with the hook.
+        solid (Solid): The solid instance associated with the hook.
+        resources (Any): Resources available in the hook context.
+        solid_config (Dict[str, Any]): The parsed config specific to this solid.
+    """
 
     def __init__(self, execution_context_data, log_manager, hook_def, step):
         from dagster.core.execution.plan.objects import ExecutionStep
 
         super(HookContext, self).__init__(execution_context_data, log_manager)
         self._log_manager = log_manager
-        self._hook_def = check.inst_param(hook_def, 'hook_def', HookDefinition)
-        self._step = check.inst_param(step, 'step', ExecutionStep)
+        self._hook_def = check.inst_param(hook_def, "hook_def", HookDefinition)
+        self._step = check.inst_param(step, "step", ExecutionStep)
 
         self._required_resource_keys = hook_def.required_resource_keys
         self._resources = self._execution_context_data.scoped_resources_builder.build(
@@ -349,3 +352,8 @@ class HookContext(SystemExecutionContext):
     @property
     def required_resource_keys(self):
         return self._required_resource_keys
+
+    @property
+    def solid_config(self):
+        solid_config = self.environment_config.solids.get(str(self._step.solid_handle))
+        return solid_config.config if solid_config else None
