@@ -1,8 +1,13 @@
 from abc import ABCMeta, abstractmethod, abstractproperty
+from datetime import datetime
 
 import six
 
 from dagster import check
+from dagster.api.snapshot_executable import (
+    sync_get_external_executable_params,
+    sync_get_external_executable_params_grpc,
+)
 from dagster.api.snapshot_execution_plan import sync_get_external_execution_plan_grpc
 from dagster.api.snapshot_partition import (
     sync_get_external_partition_config,
@@ -44,6 +49,7 @@ from dagster.core.snap.execution_plan_snapshot import (
 )
 from dagster.core.storage.pipeline_run import PipelineRun
 from dagster.grpc.impl import (
+    get_external_executable_params,
     get_external_schedule_execution,
     get_partition_config,
     get_partition_names,
@@ -51,12 +57,14 @@ from dagster.grpc.impl import (
     get_partition_tags,
 )
 from dagster.grpc.types import (
+    ExternalExecutableArgs,
     ExternalScheduleExecutionArgs,
     PartitionArgs,
     PartitionNamesArgs,
     PartitionSetExecutionParamArgs,
     ScheduleExecutionDataMode,
 )
+from dagster.seven import get_timestamp_from_utc_datetime
 from dagster.utils.hosted_user_process import (
     external_repo_from_def,
     is_repository_location_in_same_python_env,
@@ -67,7 +75,7 @@ from .selector import PipelineSelector
 
 
 class RepositoryLocation(six.with_metaclass(ABCMeta)):
-    '''
+    """
     A RepositoryLocation represents a target containing user code which has a set of Dagster
     definition objects. A given location will contain some number of uniquely named
     RepositoryDefinitions, which therein contains Pipeline, Solid, and other definitions.
@@ -84,7 +92,7 @@ class RepositoryLocation(six.with_metaclass(ABCMeta)):
 
     We also allow for InProcessRepositoryLocation which actually loads the user-defined artifacts
     into process with the host tool. This is mostly for test scenarios.
-    '''
+    """
 
     @abstractmethod
     def get_repository(self, name):
@@ -157,10 +165,19 @@ class RepositoryLocation(six.with_metaclass(ABCMeta)):
 
     @abstractmethod
     def get_external_schedule_execution_data(
-        self, instance, repository_handle, schedule_name, schedule_execution_data_mode
+        self,
+        instance,
+        repository_handle,
+        schedule_name,
+        schedule_execution_data_mode,
+        scheduled_execution_datetime_utc,
     ):
         pass
 
+    @abstractmethod
+    def get_external_executable_params(self, instance, repository_handle, name):
+        pass
+
     @abstractproperty
     def is_reload_supported(self):
         pass
@@ -168,7 +185,7 @@ class RepositoryLocation(six.with_metaclass(ABCMeta)):
     @staticmethod
     def from_handle(repository_location_handle):
         check.inst_param(
-            repository_location_handle, 'repository_location_handle', RepositoryLocationHandle
+            repository_location_handle, "repository_location_handle", RepositoryLocationHandle
         )
 
         if isinstance(repository_location_handle, InProcessRepositoryLocationHandle):
@@ -182,7 +199,7 @@ class RepositoryLocation(six.with_metaclass(ABCMeta)):
         ) or isinstance(repository_location_handle, ManagedGrpcPythonEnvRepositoryLocationHandle):
             return GrpcServerRepositoryLocation(repository_location_handle)
         else:
-            check.failed('Unsupported handle: {}'.format(repository_location_handle))
+            check.failed("Unsupported handle: {}".format(repository_location_handle))
 
     def create_reloaded_repository_location(self):
         return RepositoryLocation.from_handle(self.location_handle.create_reloaded_handle())
@@ -190,7 +207,7 @@ class RepositoryLocation(six.with_metaclass(ABCMeta)):
 
 class InProcessRepositoryLocation(RepositoryLocation):
     def __init__(self, recon_repo):
-        self._recon_repo = check.inst_param(recon_repo, 'recon_repo', ReconstructableRepository)
+        self._recon_repo = check.inst_param(recon_repo, "recon_repo", ReconstructableRepository)
         self._handle = RepositoryLocationHandle.create_in_process_location(recon_repo.pointer)
 
         repo_def = recon_repo.get_definition()
@@ -229,10 +246,10 @@ class InProcessRepositoryLocation(RepositoryLocation):
         return self._repositories
 
     def get_subset_external_pipeline_result(self, selector):
-        check.inst_param(selector, 'selector', PipelineSelector)
+        check.inst_param(selector, "selector", PipelineSelector)
         check.invariant(
             selector.location_name == self.name,
-            'PipelineSelector location_name mismatch, got {selector.location_name} expected {self.name}'.format(
+            "PipelineSelector location_name mismatch, got {selector.location_name} expected {self.name}".format(
                 self=self, selector=selector
             ),
         )
@@ -246,10 +263,10 @@ class InProcessRepositoryLocation(RepositoryLocation):
     def get_external_execution_plan(
         self, external_pipeline, run_config, mode, step_keys_to_execute
     ):
-        check.inst_param(external_pipeline, 'external_pipeline', ExternalPipeline)
-        check.dict_param(run_config, 'run_config')
-        check.str_param(mode, 'mode')
-        check.opt_list_param(step_keys_to_execute, 'step_keys_to_execute', of_type=str)
+        check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
+        check.dict_param(run_config, "run_config")
+        check.str_param(mode, "mode")
+        check.opt_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
 
         return ExternalExecutionPlan(
             execution_plan_snapshot=snapshot_from_execution_plan(
@@ -277,11 +294,11 @@ class InProcessRepositoryLocation(RepositoryLocation):
         step_keys_to_execute,
         retries=None,
     ):
-        check.inst_param(instance, 'instance', DagsterInstance)
-        check.inst_param(external_pipeline, 'external_pipeline', ExternalPipeline)
-        check.dict_param(run_config, 'run_config')
-        check.inst_param(pipeline_run, 'pipeline_run', PipelineRun)
-        check.opt_list_param(step_keys_to_execute, 'step_keys_to_execute', of_type=str)
+        check.inst_param(instance, "instance", DagsterInstance)
+        check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
+        check.dict_param(run_config, "run_config")
+        check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
+        check.opt_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
 
         execution_plan = create_execution_plan(
             pipeline=self.get_reconstructable_pipeline(
@@ -303,9 +320,9 @@ class InProcessRepositoryLocation(RepositoryLocation):
     def execute_pipeline(
         self, instance, external_pipeline, pipeline_run,
     ):
-        check.inst_param(instance, 'instance', DagsterInstance)
-        check.inst_param(external_pipeline, 'external_pipeline', ExternalPipeline)
-        check.inst_param(pipeline_run, 'pipeline_run', PipelineRun)
+        check.inst_param(instance, "instance", DagsterInstance)
+        check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
+        check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
 
         pipeline = self.get_reconstructable_pipeline(
             external_pipeline.name
@@ -316,9 +333,9 @@ class InProcessRepositoryLocation(RepositoryLocation):
         return ExternalPipelineExecutionResult(event_list=execution_result.event_list)
 
     def get_external_partition_config(self, repository_handle, partition_set_name, partition_name):
-        check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
-        check.str_param(partition_set_name, 'partition_set_name')
-        check.str_param(partition_name, 'partition_name')
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(partition_set_name, "partition_set_name")
+        check.str_param(partition_name, "partition_name")
 
         args = PartitionArgs(
             repository_origin=repository_handle.get_origin(),
@@ -328,9 +345,9 @@ class InProcessRepositoryLocation(RepositoryLocation):
         return get_partition_config(args)
 
     def get_external_partition_tags(self, repository_handle, partition_set_name, partition_name):
-        check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
-        check.str_param(partition_set_name, 'partition_set_name')
-        check.str_param(partition_name, 'partition_name')
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(partition_set_name, "partition_set_name")
+        check.str_param(partition_name, "partition_name")
 
         args = PartitionArgs(
             repository_origin=repository_handle.get_origin(),
@@ -340,8 +357,8 @@ class InProcessRepositoryLocation(RepositoryLocation):
         return get_partition_tags(args)
 
     def get_external_partition_names(self, repository_handle, partition_set_name):
-        check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
-        check.str_param(partition_set_name, 'partition_set_name')
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(partition_set_name, "partition_set_name")
 
         args = PartitionNamesArgs(
             repository_origin=repository_handle.get_origin(), partition_set_name=partition_set_name
@@ -349,13 +366,21 @@ class InProcessRepositoryLocation(RepositoryLocation):
         return get_partition_names(args)
 
     def get_external_schedule_execution_data(
-        self, instance, repository_handle, schedule_name, schedule_execution_data_mode
+        self,
+        instance,
+        repository_handle,
+        schedule_name,
+        schedule_execution_data_mode,
+        scheduled_execution_datetime_utc,
     ):
-        check.inst_param(instance, 'instance', DagsterInstance)
-        check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
-        check.str_param(schedule_name, 'schedule_name')
+        check.inst_param(instance, "instance", DagsterInstance)
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(schedule_name, "schedule_name")
         check.inst_param(
-            schedule_execution_data_mode, 'schedule_execution_data_mode', ScheduleExecutionDataMode
+            schedule_execution_data_mode, "schedule_execution_data_mode", ScheduleExecutionDataMode
+        )
+        check.opt_inst_param(
+            scheduled_execution_datetime_utc, "scheduled_execution_datetime_utc", datetime
         )
 
         repo_origin = repository_handle.get_origin()
@@ -363,17 +388,34 @@ class InProcessRepositoryLocation(RepositoryLocation):
             instance_ref=instance.get_ref(),
             repository_origin=repo_origin,
             schedule_name=schedule_name,
+            scheduled_execution_timestamp_utc=(
+                get_timestamp_from_utc_datetime(scheduled_execution_datetime_utc)
+                if scheduled_execution_datetime_utc
+                else None
+            ),
             schedule_execution_data_mode=schedule_execution_data_mode,
         )
         recon_repo = recon_repository_from_origin(repo_origin)
         return get_external_schedule_execution(recon_repo, args)
 
+    def get_external_executable_params(self, instance, repository_handle, name):
+        check.inst_param(instance, "instance", DagsterInstance)
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(name, "name")
+
+        repo_origin = repository_handle.get_origin()
+        recon_repo = recon_repository_from_origin(repo_origin)
+        args = ExternalExecutableArgs(
+            instance_ref=instance.get_ref(), repository_origin=repo_origin, name=name,
+        )
+        return get_external_executable_params(recon_repo, args)
+
     def get_external_partition_set_execution_param_data(
         self, repository_handle, partition_set_name, partition_names
     ):
-        check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
-        check.str_param(partition_set_name, 'partition_set_name')
-        check.list_param(partition_names, 'partition_names', of_type=str)
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(partition_set_name, "partition_set_name")
+        check.list_param(partition_names, "partition_names", of_type=str)
 
         args = PartitionSetExecutionParamArgs(
             repository_origin=repository_handle.get_origin(),
@@ -388,7 +430,7 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
         check.param_invariant(
             isinstance(repository_location_handle, GrpcServerRepositoryLocationHandle)
             or isinstance(repository_location_handle, ManagedGrpcPythonEnvRepositoryLocationHandle),
-            'repository_location_handle',
+            "repository_location_handle",
         )
 
         self._handle = repository_location_handle
@@ -404,7 +446,7 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
         return True
 
     def get_repository(self, name):
-        check.str_param(name, 'name')
+        check.str_param(name, "name")
         return self.external_repositories[name]
 
     def has_repository(self, name):
@@ -424,10 +466,10 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
     def get_external_execution_plan(
         self, external_pipeline, run_config, mode, step_keys_to_execute
     ):
-        check.inst_param(external_pipeline, 'external_pipeline', ExternalPipeline)
-        check.dict_param(run_config, 'run_config')
-        check.str_param(mode, 'mode')
-        check.opt_list_param(step_keys_to_execute, 'step_keys_to_execute', of_type=str)
+        check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
+        check.dict_param(run_config, "run_config")
+        check.str_param(mode, "mode")
+        check.opt_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
 
         execution_plan_snapshot_or_error = sync_get_external_execution_plan_grpc(
             api_client=self._handle.client,
@@ -456,26 +498,26 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
         step_keys_to_execute,
         retries=None,
     ):
-        raise NotImplementedError('execute_plan is not implemented for grpc servers')
+        raise NotImplementedError("execute_plan is not implemented for grpc servers")
 
     def execute_pipeline(
         self, instance, external_pipeline, pipeline_run,
     ):
         from dagster.api.execute_run import sync_execute_run_grpc
 
-        check.inst_param(instance, 'instance', DagsterInstance)
-        check.inst_param(external_pipeline, 'external_pipeline', ExternalPipeline)
-        check.inst_param(pipeline_run, 'pipeline_run', PipelineRun)
+        check.inst_param(instance, "instance", DagsterInstance)
+        check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
+        check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
         event_list = sync_execute_run_grpc(
             self._handle.client, instance.get_ref(), external_pipeline.get_origin(), pipeline_run
         )
         return ExternalPipelineExecutionResult(event_list=event_list)
 
     def get_subset_external_pipeline_result(self, selector):
-        check.inst_param(selector, 'selector', PipelineSelector)
+        check.inst_param(selector, "selector", PipelineSelector)
         check.invariant(
             selector.location_name == self.name,
-            'PipelineSelector location_name mismatch, got {selector.location_name} expected {self.name}'.format(
+            "PipelineSelector location_name mismatch, got {selector.location_name} expected {self.name}".format(
                 self=self, selector=selector
             ),
         )
@@ -487,39 +529,47 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
         )
 
     def get_external_partition_config(self, repository_handle, partition_set_name, partition_name):
-        check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
-        check.str_param(partition_set_name, 'partition_set_name')
-        check.str_param(partition_name, 'partition_name')
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(partition_set_name, "partition_set_name")
+        check.str_param(partition_name, "partition_name")
 
         return sync_get_external_partition_config_grpc(
             self._handle.client, repository_handle, partition_set_name, partition_name
         )
 
     def get_external_partition_tags(self, repository_handle, partition_set_name, partition_name):
-        check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
-        check.str_param(partition_set_name, 'partition_set_name')
-        check.str_param(partition_name, 'partition_name')
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(partition_set_name, "partition_set_name")
+        check.str_param(partition_name, "partition_name")
 
         return sync_get_external_partition_tags_grpc(
             self._handle.client, repository_handle, partition_set_name, partition_name
         )
 
     def get_external_partition_names(self, repository_handle, partition_set_name):
-        check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
-        check.str_param(partition_set_name, 'partition_set_name')
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(partition_set_name, "partition_set_name")
 
         return sync_get_external_partition_names_grpc(
             self._handle.client, repository_handle, partition_set_name
         )
 
     def get_external_schedule_execution_data(
-        self, instance, repository_handle, schedule_name, schedule_execution_data_mode
+        self,
+        instance,
+        repository_handle,
+        schedule_name,
+        schedule_execution_data_mode,
+        scheduled_execution_datetime_utc,
     ):
-        check.inst_param(instance, 'instance', DagsterInstance)
-        check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
-        check.str_param(schedule_name, 'schedule_name')
+        check.inst_param(instance, "instance", DagsterInstance)
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(schedule_name, "schedule_name")
         check.inst_param(
-            schedule_execution_data_mode, 'schedule_execution_data_mode', ScheduleExecutionDataMode
+            schedule_execution_data_mode, "schedule_execution_data_mode", ScheduleExecutionDataMode
+        )
+        check.opt_inst_param(
+            scheduled_execution_datetime_utc, "scheduled_execution_datetime_utc", datetime
         )
 
         return sync_get_external_schedule_execution_data_grpc(
@@ -528,14 +578,23 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
             repository_handle,
             schedule_name,
             schedule_execution_data_mode,
+            scheduled_execution_datetime_utc,
+        )
+
+    def get_external_executable_params(self, instance, repository_handle, name):
+        check.inst_param(instance, "instance", DagsterInstance)
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(name, "name")
+        return sync_get_external_executable_params_grpc(
+            self._handle.client, instance, repository_handle, name
         )
 
     def get_external_partition_set_execution_param_data(
         self, repository_handle, partition_set_name, partition_names
     ):
-        check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
-        check.str_param(partition_set_name, 'partition_set_name')
-        check.list_param(partition_names, 'partition_names', of_type=str)
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(partition_set_name, "partition_set_name")
+        check.list_param(partition_names, "partition_names", of_type=str)
 
         return sync_get_external_partition_set_execution_param_data_grpc(
             self._handle.client, repository_handle, partition_set_name, partition_names
@@ -546,7 +605,7 @@ class PythonEnvRepositoryLocation(RepositoryLocation):
     def __init__(self, repository_location_handle):
         self._handle = check.inst_param(
             repository_location_handle,
-            'repository_location_handle',
+            "repository_location_handle",
             PythonEnvRepositoryLocationHandle,
         )
 
@@ -558,7 +617,7 @@ class PythonEnvRepositoryLocation(RepositoryLocation):
         return True
 
     def get_repository(self, name):
-        check.str_param(name, 'name')
+        check.str_param(name, "name")
         return self.external_repositories[name]
 
     def has_repository(self, name):
@@ -580,10 +639,10 @@ class PythonEnvRepositoryLocation(RepositoryLocation):
     ):
         from dagster.api.snapshot_execution_plan import sync_get_external_execution_plan
 
-        check.inst_param(external_pipeline, 'external_pipeline', ExternalPipeline)
-        check.dict_param(run_config, 'run_config')
-        check.str_param(mode, 'mode')
-        check.opt_list_param(step_keys_to_execute, 'step_keys_to_execute', of_type=str)
+        check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
+        check.dict_param(run_config, "run_config")
+        check.str_param(mode, "mode")
+        check.opt_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
 
         execution_plan_snapshot_or_error = sync_get_external_execution_plan(
             pipeline_origin=external_pipeline.get_origin(),
@@ -615,11 +674,11 @@ class PythonEnvRepositoryLocation(RepositoryLocation):
             is_repository_location_in_same_python_env(self.location_handle)
             and len(self.location_handle.repository_code_pointer_dict) == 1
         ):
-            check.inst_param(instance, 'instance', DagsterInstance)
-            check.inst_param(external_pipeline, 'external_pipeline', ExternalPipeline)
-            check.dict_param(run_config, 'run_config')
-            check.inst_param(pipeline_run, 'pipeline_run', PipelineRun)
-            check.opt_list_param(step_keys_to_execute, 'step_keys_to_execute', of_type=str)
+            check.inst_param(instance, "instance", DagsterInstance)
+            check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
+            check.dict_param(run_config, "run_config")
+            check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
+            check.opt_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
 
             pointer = next(iter(self.location_handle.repository_code_pointer_dict.values()))
             recon_repo = ReconstructableRepository(pointer)
@@ -642,9 +701,9 @@ class PythonEnvRepositoryLocation(RepositoryLocation):
             )
         else:
             raise NotImplementedError(
-                'execute_plan is currently only supported when the location is a python '
-                'environment with the exact same executable and when there is only a single '
-                'repository.'
+                "execute_plan is currently only supported when the location is a python "
+                "environment with the exact same executable and when there is only a single "
+                "repository."
             )
 
     def execute_pipeline(
@@ -652,9 +711,9 @@ class PythonEnvRepositoryLocation(RepositoryLocation):
     ):
         from dagster.api.execute_run import cli_api_execute_run
 
-        check.inst_param(instance, 'instance', DagsterInstance)
-        check.inst_param(external_pipeline, 'external_pipeline', ExternalPipeline)
-        check.inst_param(pipeline_run, 'pipeline_run', PipelineRun)
+        check.inst_param(instance, "instance", DagsterInstance)
+        check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
+        check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
 
         event_list = cli_api_execute_run(
             instance=instance,
@@ -667,10 +726,10 @@ class PythonEnvRepositoryLocation(RepositoryLocation):
     def get_subset_external_pipeline_result(self, selector):
         from dagster.api.snapshot_pipeline import sync_get_external_pipeline_subset
 
-        check.inst_param(selector, 'selector', PipelineSelector)
+        check.inst_param(selector, "selector", PipelineSelector)
         check.invariant(
             selector.location_name == self.name,
-            'PipelineSelector location_name mismatch, got {selector.location_name} expected {self.name}'.format(
+            "PipelineSelector location_name mismatch, got {selector.location_name} expected {self.name}".format(
                 self=self, selector=selector
             ),
         )
@@ -682,49 +741,67 @@ class PythonEnvRepositoryLocation(RepositoryLocation):
         )
 
     def get_external_partition_config(self, repository_handle, partition_set_name, partition_name):
-        check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
-        check.str_param(partition_set_name, 'partition_set_name')
-        check.str_param(partition_name, 'partition_name')
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(partition_set_name, "partition_set_name")
+        check.str_param(partition_name, "partition_name")
 
         return sync_get_external_partition_config(
             repository_handle, partition_set_name, partition_name
         )
 
     def get_external_partition_tags(self, repository_handle, partition_set_name, partition_name):
-        check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
-        check.str_param(partition_set_name, 'partition_set_name')
-        check.str_param(partition_name, 'partition_name')
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(partition_set_name, "partition_set_name")
+        check.str_param(partition_name, "partition_name")
 
         return sync_get_external_partition_tags(
             repository_handle, partition_set_name, partition_name
         )
 
     def get_external_partition_names(self, repository_handle, partition_set_name):
-        check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
-        check.str_param(partition_set_name, 'partition_set_name')
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(partition_set_name, "partition_set_name")
 
         return sync_get_external_partition_names(repository_handle, partition_set_name)
 
     def get_external_schedule_execution_data(
-        self, instance, repository_handle, schedule_name, schedule_execution_data_mode
+        self,
+        instance,
+        repository_handle,
+        schedule_name,
+        schedule_execution_data_mode,
+        scheduled_execution_datetime_utc,
     ):
-        check.inst_param(instance, 'instance', DagsterInstance)
-        check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
-        check.str_param(schedule_name, 'schedule_name')
+        check.inst_param(instance, "instance", DagsterInstance)
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(schedule_name, "schedule_name")
         check.inst_param(
-            schedule_execution_data_mode, 'schedule_execution_data_mode', ScheduleExecutionDataMode
+            schedule_execution_data_mode, "schedule_execution_data_mode", ScheduleExecutionDataMode
+        )
+        check.opt_inst_param(
+            scheduled_execution_datetime_utc, "scheduled_execution_datetime_utc", datetime
         )
 
         return sync_get_external_schedule_execution_data(
-            instance, repository_handle, schedule_name, schedule_execution_data_mode
+            instance,
+            repository_handle,
+            schedule_name,
+            schedule_execution_data_mode,
+            scheduled_execution_datetime_utc,
         )
 
+    def get_external_executable_params(self, instance, repository_handle, name):
+        check.inst_param(instance, "instance", DagsterInstance)
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(name, "name")
+        return sync_get_external_executable_params(instance, repository_handle, name)
+
     def get_external_partition_set_execution_param_data(
         self, repository_handle, partition_set_name, partition_names
     ):
-        check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
-        check.str_param(partition_set_name, 'partition_set_name')
-        check.list_param(partition_names, 'partition_names', of_type=str)
+        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
+        check.str_param(partition_set_name, "partition_set_name")
+        check.list_param(partition_names, "partition_names", of_type=str)
 
         return sync_get_external_partition_set_execution_param_data(
             repository_handle, partition_set_name, partition_names
