@@ -18,14 +18,18 @@ from dagster.core.errors import PartitionExecutionError, user_code_error_boundar
 from dagster.core.host_representation.external_data import (
     ExternalPartitionConfigData,
     ExternalPartitionExecutionErrorData,
+    ExternalPartitionExecutionParamData,
     ExternalPartitionNamesData,
+    ExternalPartitionSetExecutionParamData,
     ExternalPartitionTagsData,
     external_repository_data_from_def,
 )
+from dagster.core.instance import DagsterInstance
 from dagster.core.origin import PipelineOrigin, RepositoryGrpcServerOrigin, RepositoryOrigin
+from dagster.core.types.loadable_target_origin import LoadableTargetOrigin
 from dagster.serdes import deserialize_json_to_dagster_namedtuple, serialize_dagster_namedtuple
 from dagster.serdes.ipc import IPCErrorMessage, open_ipc_subprocess
-from dagster.seven import multiprocessing
+from dagster.seven import kill_process, multiprocessing
 from dagster.utils import find_free_port, safe_tempfile_path_unmanaged
 from dagster.utils.error import serializable_error_info_from_exc_info
 from dagster.utils.hosted_user_process import recon_repository_from_origin
@@ -36,6 +40,7 @@ from .impl import (
     RunInSubprocessComplete,
     StartRunInSubprocessSuccessful,
     execute_run_in_subprocess,
+    get_external_executable_params,
     get_external_execution_plan_snapshot,
     get_external_pipeline_subset_result,
     get_external_schedule_execution,
@@ -48,13 +53,14 @@ from .types import (
     CancelExecutionResult,
     ExecuteRunArgs,
     ExecutionPlanSnapshotArgs,
+    ExternalExecutableArgs,
     ExternalScheduleExecutionArgs,
     GetCurrentImageResult,
     ListRepositoriesResponse,
     LoadableRepositorySymbol,
-    LoadableTargetOrigin,
     PartitionArgs,
     PartitionNamesArgs,
+    PartitionSetExecutionParamArgs,
     PipelineSubsetSnapshotArgs,
     ShutdownServerResult,
     StartRunResult,
@@ -63,28 +69,27 @@ from .utils import get_loadable_targets
 
 EVENT_QUEUE_POLL_INTERVAL = 0.1
 
+CLEANUP_TICK = 0.5
+
+# How long an execution process has to clean itself after termination before we hard-kill
+# the process
+GRACE_PERIOD_AFTER_PROCESS_TERMINATION = 15
+
 
 class CouldNotBindGrpcServerToAddress(Exception):
     pass
 
 
-def heartbeat_thread(heartbeat_timeout, last_heartbeat_time, shutdown_event):
-    while True:
-        time.sleep(heartbeat_timeout)
-        if last_heartbeat_time < time.time() - heartbeat_timeout:
-            shutdown_event.set()
-
-
 class LazyRepositorySymbolsAndCodePointers:
-    '''Enables lazily loading user code at RPC-time so that it doesn't interrupt startup and
-    we can gracefully handle user code errors.'''
+    """Enables lazily loading user code at RPC-time so that it doesn't interrupt startup and
+    we can gracefully handle user code errors."""
 
     def __init__(self, loadable_target_origin):
         self._loadable_target_origin = loadable_target_origin
         self._loadable_repository_symbols = None
         self._code_pointers_by_repo_name = None
 
-    def _load(self):
+    def load(self):
         self._loadable_repository_symbols = load_loadable_repository_symbols(
             self._loadable_target_origin
         )
@@ -95,14 +100,14 @@ class LazyRepositorySymbolsAndCodePointers:
     @property
     def loadable_repository_symbols(self):
         if self._loadable_repository_symbols is None:
-            self._load()
+            self.load()
 
         return self._loadable_repository_symbols
 
     @property
     def code_pointers_by_repo_name(self):
         if self._code_pointers_by_repo_name is None:
-            self._load()
+            self.load()
 
         return self._code_pointers_by_repo_name
 
@@ -155,51 +160,142 @@ class DagsterApiServer(DagsterApiServicer):
     # the target passed in here instead of passing in a target in the argument.
     def __init__(
         self,
-        shutdown_server_event,
+        server_termination_event,
         loadable_target_origin=None,
         heartbeat=False,
         heartbeat_timeout=30,
+        lazy_load_user_code=False,
     ):
         super(DagsterApiServer, self).__init__()
 
-        check.bool_param(heartbeat, 'heartbeat')
-        check.int_param(heartbeat_timeout, 'heartbeat_timeout')
-        check.invariant(heartbeat_timeout > 0, 'heartbeat_timeout must be greater than 0')
+        check.bool_param(heartbeat, "heartbeat")
+        check.int_param(heartbeat_timeout, "heartbeat_timeout")
+        check.invariant(heartbeat_timeout > 0, "heartbeat_timeout must be greater than 0")
 
-        self._shutdown_server_event = check.inst_param(
-            shutdown_server_event, 'shutdown_server_event', seven.ThreadingEventType
+        self._server_termination_event = check.inst_param(
+            server_termination_event, "server_termination_event", seven.ThreadingEventType
         )
         self._loadable_target_origin = check.opt_inst_param(
-            loadable_target_origin, 'loadable_target_origin', LoadableTargetOrigin
+            loadable_target_origin, "loadable_target_origin", LoadableTargetOrigin
         )
 
-        self._shutdown_server_event = check.inst_param(
-            shutdown_server_event, 'shutdown_server_event', seven.ThreadingEventType
-        )
-        # Dict[str, multiprocessing.Process] of run_id to execute_run process
+        # Client tells the server to shutdown by calling ShutdownServer (or by failing to send a
+        # hearbeat, at which point this event is set. The cleanup thread will then set the server
+        # termination event once all current executions have finished, which will stop the server)
+        self._shutdown_once_executions_finish_event = threading.Event()
+
+        # Dict[str, (multiprocessing.Process, DagsterInstance)]
         self._executions = {}
         # Dict[str, multiprocessing.Event]
         self._termination_events = {}
+        self._termination_times = {}
         self._execution_lock = threading.Lock()
 
         self._repository_symbols_and_code_pointers = LazyRepositorySymbolsAndCodePointers(
             loadable_target_origin
         )
+        if not lazy_load_user_code:
+            self._repository_symbols_and_code_pointers.load()
 
         self.__last_heartbeat_time = time.time()
         if heartbeat:
             self.__heartbeat_thread = threading.Thread(
-                target=heartbeat_thread,
-                args=(heartbeat_timeout, self.__last_heartbeat_time, self._shutdown_server_event),
+                target=self._heartbeat_thread, args=(heartbeat_timeout,),
             )
             self.__heartbeat_thread.daemon = True
             self.__heartbeat_thread.start()
         else:
             self.__heartbeat_thread = None
 
+        self.__cleanup_thread = threading.Thread(target=self._cleanup_thread, args=(),)
+        self.__cleanup_thread.daemon = True
+
+        self.__cleanup_thread.start()
+
+    def cleanup(self):
+        if self.__heartbeat_thread:
+            self.__heartbeat_thread.join()
+        self.__cleanup_thread.join()
+
+    def _heartbeat_thread(self, heartbeat_timeout):
+        while True:
+            self._shutdown_once_executions_finish_event.wait(heartbeat_timeout)
+            if self._shutdown_once_executions_finish_event.is_set():
+                break
+
+            if self.__last_heartbeat_time < time.time() - heartbeat_timeout:
+                self._shutdown_once_executions_finish_event.set()
+
+    def _cleanup_thread(self):
+        while True:
+            self._server_termination_event.wait(CLEANUP_TICK)
+            if self._server_termination_event.is_set():
+                break
+
+            self._check_for_orphaned_runs()
+
+    def _check_for_orphaned_runs(self):
+        with self._execution_lock:
+            runs_to_clear = []
+            for run_id, (process, instance_ref) in self._executions.items():
+                if process.is_alive():
+                    if not self._termination_events[run_id].is_set() or (
+                        time.time() - self._termination_times[run_id]
+                        < GRACE_PERIOD_AFTER_PROCESS_TERMINATION
+                    ):
+                        continue
+
+                    # the process failed to handle the KeyboardInterrupt cleanly after termination.
+                    # Inform the system and mark the run as failed.
+                    kill_process(process)
+
+                    with DagsterInstance.from_ref(instance_ref) as instance:
+                        runs_to_clear.append(run_id)
+                        run = instance.get_run_by_id(run_id)
+                        if not run:
+                            continue
+
+                        message = (
+                            "Pipeline execution process for {run_id} killed "
+                            + "after failing to terminate"
+                        ).format(run_id=run.run_id)
+                        instance.report_engine_event(message, run, cls=self.__class__)
+
+                        if run.is_finished:
+                            continue
+
+                        instance.report_run_failed(run)
+
+                else:
+                    with DagsterInstance.from_ref(instance_ref) as instance:
+                        runs_to_clear.append(run_id)
+
+                        run = instance.get_run_by_id(run_id)
+                        if not run or run.is_finished:
+                            continue
+
+                        # the process died in an unexpected manner. inform the system
+                        message = "Pipeline execution process for {run_id} unexpectedly exited.".format(
+                            run_id=run.run_id
+                        )
+                        instance.report_engine_event(message, run, cls=self.__class__)
+                        instance.report_run_failed(run)
+
+            for run_id in runs_to_clear:
+                del self._executions[run_id]
+                del self._termination_events[run_id]
+                if run_id in self._termination_times:
+                    del self._termination_times[run_id]
+
+            # Once there are no more running executions after we have received a request to
+            # shut down, terminate the server
+            if self._shutdown_once_executions_finish_event.is_set():
+                if len(self._executions) == 0:
+                    self._server_termination_event.set()
+
     def _recon_repository_from_origin(self, repository_origin):
         check.inst_param(
-            repository_origin, 'repository_origin', RepositoryOrigin,
+            repository_origin, "repository_origin", RepositoryOrigin,
         )
 
         if isinstance(repository_origin, RepositoryGrpcServerOrigin):
@@ -211,7 +307,7 @@ class DagsterApiServer(DagsterApiServicer):
         return recon_repository_from_origin(repository_origin)
 
     def _recon_pipeline_from_origin(self, pipeline_origin):
-        check.inst_param(pipeline_origin, 'pipeline_origin', PipelineOrigin)
+        check.inst_param(pipeline_origin, "pipeline_origin", PipelineOrigin)
         recon_repo = self._recon_repository_from_origin(pipeline_origin.repository_origin)
         return recon_repo.get_reconstructable_pipeline(pipeline_origin.pipeline_name)
 
@@ -235,7 +331,7 @@ class DagsterApiServer(DagsterApiServicer):
             request.serialized_execution_plan_snapshot_args
         )
 
-        check.inst_param(execution_plan_args, 'execution_plan_args', ExecutionPlanSnapshotArgs)
+        check.inst_param(execution_plan_args, "execution_plan_args", ExecutionPlanSnapshotArgs)
         recon_pipeline = self._recon_pipeline_from_origin(execution_plan_args.pipeline_origin)
         execution_plan_snapshot_or_error = get_external_execution_plan_snapshot(
             recon_pipeline, execution_plan_args
@@ -269,7 +365,7 @@ class DagsterApiServer(DagsterApiServicer):
             request.serialized_partition_names_args
         )
 
-        check.inst_param(partition_names_args, 'partition_names_args', PartitionNamesArgs)
+        check.inst_param(partition_names_args, "partition_names_args", PartitionNamesArgs)
 
         recon_repo = self._recon_repository_from_origin(partition_names_args.repository_origin)
         definition = recon_repo.get_definition()
@@ -279,8 +375,8 @@ class DagsterApiServer(DagsterApiServicer):
         try:
             with user_code_error_boundary(
                 PartitionExecutionError,
-                lambda: 'Error occurred during the execution of the partition generation function for '
-                'partition set {partition_set_name}'.format(
+                lambda: "Error occurred during the execution of the partition generation function for "
+                "partition set {partition_set_name}".format(
                     partition_set_name=partition_set_def.name
                 ),
             ):
@@ -300,10 +396,79 @@ class DagsterApiServer(DagsterApiServicer):
                 )
             )
 
+    def ExternalPartitionSetExecutionParams(self, request, _context):
+        partition_set_execution_param_args = deserialize_json_to_dagster_namedtuple(
+            request.serialized_partition_set_execution_param_args
+        )
+
+        check.inst_param(
+            partition_set_execution_param_args,
+            "partition_set_execution_param_args",
+            PartitionSetExecutionParamArgs,
+        )
+
+        recon_repo = self._recon_repository_from_origin(
+            partition_set_execution_param_args.repository_origin
+        )
+        definition = recon_repo.get_definition()
+        partition_set_def = definition.get_partition_set_def(
+            partition_set_execution_param_args.partition_set_name
+        )
+
+        try:
+            with user_code_error_boundary(
+                PartitionExecutionError,
+                lambda: "Error occurred during the partition generation for partition set "
+                "{partition_set_name}".format(partition_set_name=partition_set_def.name),
+            ):
+                all_partitions = partition_set_def.get_partitions()
+            partitions = [
+                partition
+                for partition in all_partitions
+                if partition.name in partition_set_execution_param_args.partition_names
+            ]
+
+            partition_data = []
+            for partition in partitions:
+
+                def _error_message_fn(partition_set_name, partition_name):
+                    return lambda: (
+                        "Error occurred during the partition config and tag generation for "
+                        "partition set {partition_set_name}::{partition_name}".format(
+                            partition_set_name=partition_set_name, partition_name=partition_name
+                        )
+                    )
+
+                with user_code_error_boundary(
+                    PartitionExecutionError,
+                    _error_message_fn(partition_set_def.name, partition.name),
+                ):
+                    run_config = partition_set_def.run_config_for_partition(partition)
+                    tags = partition_set_def.tags_for_partition(partition)
+
+                partition_data.append(
+                    ExternalPartitionExecutionParamData(
+                        name=partition.name, tags=tags, run_config=run_config,
+                    )
+                )
+
+            return api_pb2.ExternalPartitionSetExecutionParamsReply(
+                serialized_external_partition_set_execution_param_data_or_external_partition_execution_error=serialize_dagster_namedtuple(
+                    ExternalPartitionSetExecutionParamData(partition_data=partition_data)
+                )
+            )
+
+        except PartitionExecutionError:
+            return api_pb2.ExternalPartitionSetExecutionParamsReply(
+                ExternalPartitionExecutionErrorData(
+                    serializable_error_info_from_exc_info(sys.exc_info())
+                )
+            )
+
     def ExternalPartitionConfig(self, request, _context):
         partition_args = deserialize_json_to_dagster_namedtuple(request.serialized_partition_args)
 
-        check.inst_param(partition_args, 'partition_args', PartitionArgs)
+        check.inst_param(partition_args, "partition_args", PartitionArgs)
 
         recon_repo = self._recon_repository_from_origin(partition_args.repository_origin)
         definition = recon_repo.get_definition()
@@ -312,8 +477,8 @@ class DagsterApiServer(DagsterApiServicer):
         try:
             with user_code_error_boundary(
                 PartitionExecutionError,
-                lambda: 'Error occurred during the evaluation of the `run_config_for_partition` '
-                'function for partition set {partition_set_name}'.format(
+                lambda: "Error occurred during the evaluation of the `run_config_for_partition` "
+                "function for partition set {partition_set_name}".format(
                     partition_set_name=partition_set_def.name
                 ),
             ):
@@ -335,7 +500,7 @@ class DagsterApiServer(DagsterApiServicer):
     def ExternalPartitionTags(self, request, _context):
         partition_args = deserialize_json_to_dagster_namedtuple(request.serialized_partition_args)
 
-        check.inst_param(partition_args, 'partition_args', PartitionArgs)
+        check.inst_param(partition_args, "partition_args", PartitionArgs)
 
         recon_repo = self._recon_repository_from_origin(partition_args.repository_origin)
         definition = recon_repo.get_definition()
@@ -344,8 +509,8 @@ class DagsterApiServer(DagsterApiServicer):
         try:
             with user_code_error_boundary(
                 PartitionExecutionError,
-                lambda: 'Error occurred during the evaluation of the `tags_for_partition` function for '
-                'partition set {partition_set_name}'.format(
+                lambda: "Error occurred during the evaluation of the `tags_for_partition` function for "
+                "partition set {partition_set_name}".format(
                     partition_set_name=partition_set_def.name
                 ),
             ):
@@ -371,7 +536,7 @@ class DagsterApiServer(DagsterApiServicer):
 
         check.inst_param(
             pipeline_subset_snapshot_args,
-            'pipeline_subset_snapshot_args',
+            "pipeline_subset_snapshot_args",
             PipelineSubsetSnapshotArgs,
         )
 
@@ -389,7 +554,7 @@ class DagsterApiServer(DagsterApiServicer):
             request.serialized_repository_python_origin
         )
 
-        check.inst_param(repository_origin, 'repository_origin', RepositoryOrigin)
+        check.inst_param(repository_origin, "repository_origin", RepositoryOrigin)
 
         recon_repo = self._recon_repository_from_origin(repository_origin)
         return api_pb2.ExternalRepositoryReply(
@@ -405,7 +570,7 @@ class DagsterApiServer(DagsterApiServicer):
 
         check.inst_param(
             external_schedule_execution_args,
-            'external_schedule_execution_args',
+            "external_schedule_execution_args",
             ExternalScheduleExecutionArgs,
         )
 
@@ -419,12 +584,37 @@ class DagsterApiServer(DagsterApiServicer):
             )
         )
 
+    def ExternalExecutableParams(self, request, _context):
+        external_executable_args = deserialize_json_to_dagster_namedtuple(
+            request.serialized_external_executable_args
+        )
+        check.inst_param(
+            external_executable_args, "external_executable_args", ExternalExecutableArgs,
+        )
+
+        recon_repo = self._recon_repository_from_origin(external_executable_args.repository_origin)
+        return api_pb2.ExternalExecutableParamsReply(
+            serialized_external_execution_params_or_external_execution_params_error_data=serialize_dagster_namedtuple(
+                get_external_executable_params(recon_repo, external_executable_args)
+            )
+        )
+
     def ExecuteRun(self, request, _context):
+        if self._shutdown_once_executions_finish_event.is_set():
+            yield api_pb2.ExecuteRunEvent(
+                serialized_dagster_event_or_ipc_error_message=serialize_dagster_namedtuple(
+                    IPCErrorMessage(
+                        serializable_error_info=None,
+                        message="Tried to start a run on a server after telling it to shut down",
+                    )
+                )
+            )
+
         try:
             execute_run_args = deserialize_json_to_dagster_namedtuple(
                 request.serialized_execute_run_args
             )
-            check.inst_param(execute_run_args, 'execute_run_args', ExecuteRunArgs)
+            check.inst_param(execute_run_args, "execute_run_args", ExecuteRunArgs)
 
             run_id = execute_run_args.pipeline_run_id
 
@@ -437,7 +627,7 @@ class DagsterApiServer(DagsterApiServicer):
                         serializable_error_info=serializable_error_info_from_exc_info(
                             sys.exc_info()
                         ),
-                        message='Error during RPC setup for ExecuteRun',
+                        message="Error during RPC setup for ExecuteRun",
                     )
                 )
             )
@@ -456,7 +646,10 @@ class DagsterApiServer(DagsterApiServicer):
         )
         with self._execution_lock:
             execution_process.start()
-            self._executions[run_id] = execution_process
+            self._executions[run_id] = (
+                execution_process,
+                execute_run_args.instance_ref,
+            )
             self._termination_events[run_id] = termination_event
 
         done = False
@@ -475,7 +668,7 @@ class DagsterApiServer(DagsterApiServicer):
                                     sys.exc_info()
                                 ),
                                 message=(
-                                    'GRPC server: Subprocess for {run_id} terminated unexpectedly'
+                                    "GRPC server: Subprocess for {run_id} terminated unexpectedly"
                                 ).format(run_id=run_id),
                             )
                         )
@@ -504,7 +697,7 @@ class DagsterApiServer(DagsterApiServicer):
 
     def ShutdownServer(self, request, _context):
         try:
-            self._shutdown_server_event.set()
+            self._shutdown_once_executions_finish_event.set()
             return api_pb2.ShutdownServerReply(
                 serialized_shutdown_server_result=serialize_dagster_namedtuple(
                     ShutdownServerResult(success=True, serializable_error_info=None)
@@ -534,6 +727,7 @@ class DagsterApiServer(DagsterApiServicer):
             with self._execution_lock:
                 if cancel_execution_request.run_id in self._executions:
                     self._termination_events[cancel_execution_request.run_id].set()
+                    self._termination_times[cancel_execution_request.run_id] = time.time()
                     success = True
 
         except:  # pylint: disable=bare-except
@@ -555,7 +749,10 @@ class DagsterApiServer(DagsterApiServicer):
             CanCancelExecutionRequest,
         )
         with self._execution_lock:
-            can_cancel = can_cancel_execution_request.run_id in self._executions
+            run_id = can_cancel_execution_request.run_id
+            can_cancel = (
+                run_id in self._executions and not self._termination_events[run_id].is_set()
+            )
 
         return api_pb2.CanCancelExecutionReply(
             serialized_can_cancel_execution_result=serialize_dagster_namedtuple(
@@ -564,19 +761,23 @@ class DagsterApiServer(DagsterApiServicer):
         )
 
     def StartRun(self, request, _context):
-        execute_run_args = check.inst(
-            deserialize_json_to_dagster_namedtuple(request.serialized_execute_run_args),
-            ExecuteRunArgs,
-        )
+        if self._shutdown_once_executions_finish_event.is_set():
+            return api_pb2.StartRunReply(
+                serialized_start_run_result=serialize_dagster_namedtuple(
+                    StartRunResult(
+                        success=False,
+                        message="Tried to start a run on a server after telling it to shut down",
+                        serializable_error_info=None,
+                    )
+                )
+            )
 
         try:
             execute_run_args = check.inst(
                 deserialize_json_to_dagster_namedtuple(request.serialized_execute_run_args),
                 ExecuteRunArgs,
             )
-
             run_id = execute_run_args.pipeline_run_id
-
             recon_pipeline = self._recon_pipeline_from_origin(execute_run_args.pipeline_origin)
 
         except:  # pylint: disable=bare-except
@@ -603,9 +804,13 @@ class DagsterApiServer(DagsterApiServicer):
                 termination_event,
             ],
         )
+
         with self._execution_lock:
             execution_process.start()
-            self._executions[run_id] = execution_process
+            self._executions[run_id] = (
+                execution_process,
+                execute_run_args.instance_ref,
+            )
             self._termination_events[run_id] = termination_event
 
         success = None
@@ -623,8 +828,8 @@ class DagsterApiServer(DagsterApiServicer):
                     # subprocess died unexpectedly
                     success = False
                     message = (
-                        'GRPC server: Subprocess for {run_id} terminated unexpectedly with '
-                        'exit code {exit_code}'.format(
+                        "GRPC server: Subprocess for {run_id} terminated unexpectedly with "
+                        "exit code {exit_code}".format(
                             run_id=run_id, exit_code=execution_process.exitcode,
                         )
                     )
@@ -645,6 +850,11 @@ class DagsterApiServer(DagsterApiServicer):
                         dagster_event_or_ipc_error_message_or_done.serializable_error_info
                     )
 
+        # Ensure that if the run failed, we remove it from the executions map before
+        # returning so that CanCancel will never return True
+        if not success:
+            self._check_for_orphaned_runs()
+
         return api_pb2.StartRunReply(
             serialized_start_run_result=serialize_dagster_namedtuple(
                 StartRunResult(
@@ -656,11 +866,11 @@ class DagsterApiServer(DagsterApiServicer):
         )
 
     def GetCurrentImage(self, request, _context):
-        current_image = os.getenv('DAGSTER_CURRENT_IMAGE')
+        current_image = os.getenv("DAGSTER_CURRENT_IMAGE")
         serializable_error_info = None
 
         if not current_image:
-            serializable_error_info = 'DAGSTER_CURRENT_IMAGE is not set.'
+            serializable_error_info = "DAGSTER_CURRENT_IMAGE is not set."
         return api_pb2.GetCurrentImageReply(
             serialized_current_image=serialize_dagster_namedtuple(
                 GetCurrentImageResult(
@@ -673,73 +883,74 @@ class DagsterApiServer(DagsterApiServicer):
 # This is not a splendid scheme. We could possibly use a sentinel file for this, or send a custom
 # signal back to the client process (Unix only, i think, and questionable); or maybe the client
 # could poll the ping rpc instead/in addition to this
-SERVER_STARTED_TOKEN = 'dagster_grpc_server_started'
+SERVER_STARTED_TOKEN = "dagster_grpc_server_started"
 
-SERVER_STARTED_TOKEN_BYTES = b'dagster_grpc_server_started'
+SERVER_STARTED_TOKEN_BYTES = b"dagster_grpc_server_started"
 
-SERVER_FAILED_TO_BIND_TOKEN = 'dagster_grpc_server_failed_to_bind'
+SERVER_FAILED_TO_BIND_TOKEN = "dagster_grpc_server_failed_to_bind"
 
-SERVER_FAILED_TO_BIND_TOKEN_BYTES = b'dagster_grpc_server_failed_to_bind'
+SERVER_FAILED_TO_BIND_TOKEN_BYTES = b"dagster_grpc_server_failed_to_bind"
 
 
 def server_termination_target(termination_event, server):
-    while not termination_event.is_set():
-        time.sleep(0.1)
+    termination_event.wait()
     # We could make this grace period configurable if we set it in the ShutdownServer handler
-    server.stop(grace=None)
+    server.stop(grace=5)
 
 
 class DagsterGrpcServer(object):
     def __init__(
         self,
-        host='localhost',
+        host="localhost",
         port=None,
         socket=None,
         max_workers=1,
         loadable_target_origin=None,
         heartbeat=False,
         heartbeat_timeout=30,
+        lazy_load_user_code=False,
     ):
-        check.opt_str_param(host, 'host')
-        check.opt_int_param(port, 'port')
-        check.opt_str_param(socket, 'socket')
-        check.int_param(max_workers, 'max_workers')
-        check.opt_inst_param(loadable_target_origin, 'loadable_target_origin', LoadableTargetOrigin)
+        check.opt_str_param(host, "host")
+        check.opt_int_param(port, "port")
+        check.opt_str_param(socket, "socket")
+        check.int_param(max_workers, "max_workers")
+        check.opt_inst_param(loadable_target_origin, "loadable_target_origin", LoadableTargetOrigin)
         check.invariant(
             port is not None if seven.IS_WINDOWS else True,
-            'You must pass a valid `port` on Windows: `socket` not supported.',
+            "You must pass a valid `port` on Windows: `socket` not supported.",
         )
         check.invariant(
             (port or socket) and not (port and socket),
-            'You must pass one and only one of `port` or `socket`.',
+            "You must pass one and only one of `port` or `socket`.",
         )
         check.invariant(
-            host is not None if port else True, 'Must provide a host when serving on a port',
+            host is not None if port else True, "Must provide a host when serving on a port",
         )
-        check.bool_param(heartbeat, 'heartbeat')
-        check.int_param(heartbeat_timeout, 'heartbeat_timeout')
-        check.invariant(heartbeat_timeout > 0, 'heartbeat_timeout must be greater than 0')
+        check.bool_param(heartbeat, "heartbeat")
+        check.int_param(heartbeat_timeout, "heartbeat_timeout")
+        check.invariant(heartbeat_timeout > 0, "heartbeat_timeout must be greater than 0")
         check.invariant(
             max_workers > 1 if heartbeat else True,
             "max_workers must be greater than 1 if heartbeat is True",
         )
 
         self.server = grpc.server(ThreadPoolExecutor(max_workers=max_workers))
-        self._shutdown_server_event = threading.Event()
-        add_DagsterApiServicer_to_server(
-            DagsterApiServer(
-                shutdown_server_event=self._shutdown_server_event,
-                loadable_target_origin=loadable_target_origin,
-                heartbeat=heartbeat,
-                heartbeat_timeout=heartbeat_timeout,
-            ),
-            self.server,
+        self._server_termination_event = threading.Event()
+
+        self._servicer = DagsterApiServer(
+            server_termination_event=self._server_termination_event,
+            loadable_target_origin=loadable_target_origin,
+            heartbeat=heartbeat,
+            heartbeat_timeout=heartbeat_timeout,
+            lazy_load_user_code=lazy_load_user_code,
         )
 
+        add_DagsterApiServicer_to_server(self._servicer, self.server)
+
         if port:
-            server_address = host + ':' + str(port)
+            server_address = host + ":" + str(port)
         else:
-            server_address = 'unix:' + os.path.abspath(socket)
+            server_address = "unix:" + os.path.abspath(socket)
 
         # grpc.Server.add_insecure_port returns:
         # - 0 on failure
@@ -781,8 +992,8 @@ class DagsterGrpcServer(object):
         sys.stdout.flush()
         server_termination_thread = threading.Thread(
             target=server_termination_target,
-            args=[self._shutdown_server_event, self.server],
-            name='grpc-server-termination',
+            args=[self._server_termination_event, self.server],
+            name="grpc-server-termination",
         )
 
         server_termination_thread.daemon = True
@@ -791,15 +1002,19 @@ class DagsterGrpcServer(object):
 
         self.server.wait_for_termination()
 
+        server_termination_thread.join()
+
+        self._servicer.cleanup()
+
 
 class CouldNotStartServerProcess(Exception):
     def __init__(self, port=None, socket=None):
         super(CouldNotStartServerProcess, self).__init__(
-            'Could not start server with '
+            "Could not start server with "
             + (
-                'port {port}'.format(port=port)
+                "port {port}".format(port=port)
                 if port is not None
-                else 'socket {socket}'.format(socket=socket)
+                else "socket {socket}".format(socket=socket)
             )
         )
 
@@ -807,7 +1022,7 @@ class CouldNotStartServerProcess(Exception):
 def wait_for_grpc_server(server_process, timeout=3):
     total_time = 0
     backoff = 0.01
-    for line in iter(server_process.stdout.readline, ''):
+    for line in iter(server_process.stdout.readline, ""):
         if line.rstrip() == SERVER_FAILED_TO_BIND_TOKEN_BYTES:
             raise CouldNotBindGrpcServerToAddress()
         elif line.rstrip() != SERVER_STARTED_TOKEN_BYTES:
@@ -818,48 +1033,56 @@ def wait_for_grpc_server(server_process, timeout=3):
                 return False
         else:
             return True
+    return False
 
 
 def open_server_process(
-    port, socket, loadable_target_origin=None, max_workers=1, heartbeat=False, heartbeat_timeout=30
+    port,
+    socket,
+    loadable_target_origin=None,
+    max_workers=1,
+    heartbeat=False,
+    heartbeat_timeout=30,
+    lazy_load_user_code=False,
 ):
-    check.invariant((port or socket) and not (port and socket), 'Set only port or socket')
-    check.opt_inst_param(loadable_target_origin, 'loadable_target_origin', LoadableTargetOrigin)
-    check.int_param(max_workers, 'max_workers')
+    check.invariant((port or socket) and not (port and socket), "Set only port or socket")
+    check.opt_inst_param(loadable_target_origin, "loadable_target_origin", LoadableTargetOrigin)
+    check.int_param(max_workers, "max_workers")
 
     subprocess_args = (
         [
             loadable_target_origin.executable_path
             if loadable_target_origin and loadable_target_origin.executable_path
             else sys.executable,
-            '-m',
-            'dagster.grpc',
+            "-m",
+            "dagster.grpc",
         ]
-        + (['--port', str(port)] if port else [])
-        + (['--socket', socket] if socket else [])
-        + ['-n', str(max_workers)]
-        + (['--heartbeat'] if heartbeat else [])
-        + (['--heartbeat-timeout', str(heartbeat_timeout)] if heartbeat_timeout else [])
+        + (["--port", str(port)] if port else [])
+        + (["--socket", socket] if socket else [])
+        + ["-n", str(max_workers)]
+        + (["--heartbeat"] if heartbeat else [])
+        + (["--heartbeat-timeout", str(heartbeat_timeout)] if heartbeat_timeout else [])
+        + (["--lazy-load-user-code"] if lazy_load_user_code else [])
     )
 
     if loadable_target_origin:
         subprocess_args += (
             (
-                ['-f', loadable_target_origin.python_file]
+                ["-f", loadable_target_origin.python_file]
                 if loadable_target_origin.python_file
                 else []
             )
             + (
-                ['-m', loadable_target_origin.module_name]
+                ["-m", loadable_target_origin.module_name]
                 if loadable_target_origin.module_name
                 else []
             )
             + (
-                ['-d', loadable_target_origin.working_directory]
+                ["-d", loadable_target_origin.working_directory]
                 if loadable_target_origin.working_directory
                 else []
             )
-            + (['-a', loadable_target_origin.attribute] if loadable_target_origin.attribute else [])
+            + (["-a", loadable_target_origin.attribute] if loadable_target_origin.attribute else [])
         )
 
     server_process = open_ipc_subprocess(subprocess_args, stdout=subprocess.PIPE)
@@ -874,7 +1097,9 @@ def open_server_process(
         return None
 
 
-def open_server_process_on_dynamic_port(max_retries=10, loadable_target_origin=None, max_workers=1):
+def open_server_process_on_dynamic_port(
+    max_retries=10, loadable_target_origin=None, max_workers=1, lazy_load_user_code=False
+):
     server_process = None
     retries = 0
     while server_process is None and retries < max_retries:
@@ -885,6 +1110,7 @@ def open_server_process_on_dynamic_port(max_retries=10, loadable_target_origin=N
                 socket=None,
                 loadable_target_origin=loadable_target_origin,
                 max_workers=max_workers,
+                lazy_load_user_code=lazy_load_user_code,
             )
         except CouldNotBindGrpcServerToAddress:
             pass
@@ -913,18 +1139,20 @@ class GrpcServerProcess(object):
         max_workers=1,
         heartbeat=False,
         heartbeat_timeout=30,
+        lazy_load_user_code=False,
     ):
         self.port = None
         self.socket = None
         self.server_process = None
 
-        check.opt_inst_param(loadable_target_origin, 'loadable_target_origin', LoadableTargetOrigin)
-        check.bool_param(force_port, 'force_port')
-        check.int_param(max_retries, 'max_retries')
-        check.int_param(max_workers, 'max_workers')
-        check.bool_param(heartbeat, 'heartbeat')
-        check.int_param(heartbeat_timeout, 'heartbeat_timeout')
-        check.invariant(heartbeat_timeout > 0, 'heartbeat_timeout must be greater than 0')
+        check.opt_inst_param(loadable_target_origin, "loadable_target_origin", LoadableTargetOrigin)
+        check.bool_param(force_port, "force_port")
+        check.int_param(max_retries, "max_retries")
+        check.int_param(max_workers, "max_workers")
+        check.bool_param(heartbeat, "heartbeat")
+        check.int_param(heartbeat_timeout, "heartbeat_timeout")
+        check.invariant(heartbeat_timeout > 0, "heartbeat_timeout must be greater than 0")
+        check.bool_param(lazy_load_user_code, "lazy_load_user_code")
         check.invariant(
             max_workers > 1 if heartbeat else True,
             "max_workers must be greater than 1 if heartbeat is True",
@@ -935,9 +1163,9 @@ class GrpcServerProcess(object):
                 max_retries=max_retries,
                 loadable_target_origin=loadable_target_origin,
                 max_workers=max_workers,
+                lazy_load_user_code=lazy_load_user_code,
             )
         else:
-            # Who will clean this up now
             self.socket = safe_tempfile_path_unmanaged()
 
             self.server_process = open_server_process(
@@ -947,11 +1175,16 @@ class GrpcServerProcess(object):
                 max_workers=max_workers,
                 heartbeat=heartbeat,
                 heartbeat_timeout=heartbeat_timeout,
+                lazy_load_user_code=lazy_load_user_code,
             )
 
         if self.server_process is None:
             raise CouldNotStartServerProcess(port=self.port, socket=self.socket)
 
+    def wait(self, timeout=30):
+        if self.server_process.poll() is None:
+            seven.wait_for_process(self.server_process, timeout=timeout)
+
     def create_ephemeral_client(self):
         from dagster.grpc.client import EphemeralDagsterGrpcClient
 
